#include <cstdio>
#include "clock.h"

#include "teststatic.h"
#include "testcpp.h"
#include "teststandard.h"
#include "testdynamic.h"
#include "testdynamicindirect.h"
#include "testobjc.h"

int main(int argc, char * args[])
{
	unsigned int testCount = 400000000;
	
	unsigned int lapCount = 3;
	
	unsigned int staticTime = 0;
	unsigned int cppTime = 0;
	unsigned int standardTime = 0;
	unsigned int dynamicTime = 0;
	unsigned int dynamicIndirectTime = 0;
	unsigned int objcTime = 0;
	
	for (int i = 0; i < lapCount; ++i)
	{
		printf("== Lap %d of %d; testing %d function calls ==\n", i+1, lapCount, testCount);
		unsigned int result;
		unsigned int base;
		
		printf("Static:           ");
		result = testStatic(testCount);
		printClock(result);
		staticTime += result;
		base = result;
		
		printf("C++:              ");
		result = testCpp(testCount);
		printClockCompare(result, base);
		cppTime += result;
		
		printf("Standard:         ");
		result = testStandard(testCount);
		printClockCompare(result, base);
		standardTime += result;
		
		printf("Dynamic:          ");
		result = testDynamic(testCount);
		printClockCompare(result, base);
		dynamicTime += result;
		
		printf("Dynamic Indirect: ");
		result = testDynamicIndirect(testCount);
		printClockCompare(result, base);
		dynamicIndirectTime += result;
		
		printf("Objective-C:      ");
		result = testObjc(testCount);
		printClockCompare(result, base);
		objcTime += result;
	}
	printf("== %d-lap average; totaling %d function calls ==\n", 
		lapCount, testCount*lapCount);

	printf("Static:           %d ticks; %f (base time)\n", staticTime/lapCount, 1.0);
	printf("C++:              %d ticks; %f\n", cppTime/lapCount, (float)cppTime / staticTime);
	printf("Standard:         %d ticks; %f\n", standardTime/lapCount, (float)standardTime / staticTime);
	printf("Dynamic:          %d ticks; %f\n", dynamicTime/lapCount, (float)dynamicTime / staticTime);
	printf("Dynamic Indirect: %d ticks; %f\n", dynamicIndirectTime/lapCount, (float)dynamicIndirectTime / staticTime);
	printf("Objective-C:      %d ticks; %f\n", objcTime/lapCount, (float)objcTime / staticTime);

	return 0;
}
